@@ -64,7 +64,7 @@ class EnvironmentConfiguration(object):
         except ClientError:
             raise UnrecoverableException("Unable to fetch environment configuration from DynamoDB.")
         except KeyError:
-            raise UnrecoverableException(f"Environment configuration for {self.environment} not found. Does this environment exist?")
+            raise UnrecoverableException("Environment configuration not found. Does this environment exist?")
 
     def update_config(self):
 
@@ -180,7 +180,7 @@ class EnvironmentConfiguration(object):
             "environment": {
                 "notifications_arn": notifications_arn,
                 "ssl_certificate_arn": ssl_certificate_arn
-            }
+            },
         }, 'cloudlift_version': VERSION}
         self._set_config(environment_configuration)
         pass
